<?php

namespace Drupal\pricespider\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\pricespider\Service\PriceSpiderService;
use Drupal\Core\Form\FormStateInterface;

/**
 * Displays 'Where to Buy' Price spider functionality.
 *
 * @Block(
 *   id = "pricespider_wtb_block",
 *   admin_label = @Translation("Where to Buy"),
 * )
 */
class PriceSpiderWTBBlock extends BlockBase implements BlockPluginInterface, ContainerFactoryPluginInterface {

  /**
   * PriceSpiderService.
   *
   * @var \Drupal\pricespider\Service\PriceSpiderService
   */
  protected $priceSpiderService;

  /**
   * Config (Widget) Id configured in Pricespider settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * PriceSpiderWTBBlock constructor.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\pricespider\Service\PriceSpiderService $priceSpiderService
   *   The PriceSpider Service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The Config Factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PriceSpiderService $priceSpiderService, ConfigFactoryInterface $configFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->priceSpiderService = $priceSpiderService;
    $this->config = $configFactory->get('pricespider.settings')->get('ps.config');
  }

  /**
   * Instantiates a new instance of this class.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('pricespider'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'sku' => NULL,
      'config_id' => $this->config,
    ];
  }


  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['pricespider_default_sku'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default SKU value'),
      '#default_value' => $this->configuration['sku'],
      '#description' => $this->t('A default SKU is required to launch the embedded, full-page PriceSpider widget.'),
      '#required' => TRUE,
    ];

    $form['pricespider_config_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Config (Widget) ID)'),
      '#default_value' => $this->configuration['config_id'],
      '#description' => $this->t('Configure a PriceSpider Config (widget) ID or use the default.'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['sku'] = $form_state->getValue('pricespider_default_sku');
    $this->configuration['config_id'] = $form_state->getValue('pricespider_config_id');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [
      '#theme' => 'pricespider_wtb_block',
      '#sku' => $this->configuration['sku'],
      '#widget' => $this->configuration['config_id'],
      '#attached' => [
        'library' => ['pricespider/pricespider.js', 'pricespider/main'],
        'html_head' => $this->priceSpiderService->getMetaTags([
          'ps-account',
          'ps-language',
          'ps-country',
        ]),
      ],
    ];
    // Set cache tags based on pricespider settings changes.
    $build['#cache']['tags'][] = 'config:pricespider.settings';

    return $build;
  }

}
