<?php

namespace Drupal\pricespider\Service;

use Drupal\Core\Entity\EntityInterface;

/**
 * Interface PriceSpiderServiceInterface.
 */
interface PriceSpiderServiceInterface {

  /**
   * Returns boolean value if entity bundle is marked as a product type.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   *
   * @return bool
   *   Boolean TRUE or FALSE
   */
  public function isProductType($entity_type, $bundle);

  /**
   * Return an associative array of Entity type/bundles marked as product types.
   *
   * Array is keyed by Entity Type, followed by Bundle name, and sku
   * field name.
   *
   * @return array
   *   The content type and field name.
   */
  public function getProductTypes();

  /**
   * Remove an entity type or bundle as a product type.
   *
   * @param string $entity_type
   *   Entity type.
   * @param bool $bundle
   *   Entity bundle (optional)
   */
  public function removeProductType($entity_type, $bundle = FALSE);

  /**
   * Return associative array of fields that can be used for SKU.
   *
   * Key should be field name and value is the human readable label.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   *
   * @return array
   *   Returns associative array of fields.
   */
  public function getSkuFieldOptions($entity_type, $bundle);

  /**
   * Return the field for the entity bundle that holds the SKU value.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   *
   * @return string
   *   Name of the field
   */
  public function getSkuField($entity_type, $bundle);

  /**
   * Set the field to be used to retrieve sku for an entity bundle.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   * @param string $sku_field
   *   Sku field name.
   *
   * @return mixed
   *   Define which field to use for the sku.
   */
  public function setSkuField($entity_type, $bundle, $sku_field = '');

  /**
   * Returns the SKU value for an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity object.
   *
   * @return string
   *   Value from sku field.
   */
  public function getSkuValue(EntityInterface $entity);

  /**
   * Return a full or filtered array of Pricespider metatags.
   *
   * @param array $tag_names
   *   Array of tag names to filter.
   *
   * @return array
   *   Array of Drupal build arrays for HTML meta tags.
   */
  public function getMetaTags(array $tag_names = []);

  /**
   * Set the config override value for the entity bundle.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   * @param bool $override_value
   *   Override value.
   */
  public function setProductTypeConfigOverride($entity_type, $bundle, $override_value);

  /**
   * Get the config override value for the entity bundle.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   *
   * @return string|NULL
   *   0 or 1 when configured, else NULL.
   */
  public function getProductTypeConfigOverride($entity_type, $bundle);

  /**
   * Set Config (Widget) Ids override for the bundle.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   * @param string $widget_ids
   *   Submitted Config (Widget) Id's to override the default.
   */
  public function setProductTypeWidgetIds($entity_type, $bundle, $widget_ids);

  /**
   * Get Config (Widget) Id's overriding the entity bundle default config.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   *
   * @return string|NULL
   *   Comma separated string of config ids when configured, else NULL.
   */
  public function getProductTypeWidgetIds($entity_type, $bundle);

  /**
   * Delete Config (Widget) Ids override for the bundle.
   *
   * @param string $entity_type
   *   Entity type.
   * @param string $bundle
   *   Bundle name.
   */
  public function removeProductTypeWidgetIds($entity_type, $bundle);

}
